/******** <lalVerbatim file="BlockApplySearchCV"> ********
Author: Tibbits, M. M.
$Id: BlockApplySearch.c,v 1.1 2002/01/24 23:08:04 jolien Exp $
********* </lalVerbatim> ********/

#include "BlockApplySearch.h"
#include "BlockConditionData.h"
#include "BlockBuildOutput.h"

NRCSID( BLOCKAPPLYSEARCHC, "$Id: BlockApplySearch.c,v 1.1 2002/01/24 23:08:04 jolien Exp $");

void LALApplySearch
(
	LALStatus		*status,
	LALSearchOutput		*output,
	LALSearchInput		*input,
	LALApplySearchParams	*applyParams
)
{
	/*  Variable Declarations  */
	BlockSearchParams	*params;

	INITSTATUS( status, "BlockApplySearch", BLOCKAPPLYSEARCHC );
	ATTATCHSTATUSPTR (status);

	/*  Inititalize Variables  */
	params = ((BlockSearchParams*)(applyParams->searchParams));

	/*  Sanity Checks on Input  */
	ASSERT( output, status, BLOCKAPPLYSEARCHH_ENULL, BLOCKAPPLYSEARCHH_MSGENULL);

	ASSERT( input, status, BLOCKAPPLYSEARCHH_ENULL, BLOCKAPPLYSEARCHH_MSGENULL);

	ASSERT( applyParams, status, BLOCKAPPLYSEARCHH_ENULL, BLOCKAPPLYSEARCHH_MSGENULL);

	/*  Get rank  */
	MPI_Comm_rank(*(applyParams->mpiParams->comm), &(params->rank));

	/*  Which node is this running on?  */
	if (params->searchMaster)
	{
		/* master */
		ApplySearchMaster(status->statusPtr, output, input, applyParams );
	}
	else
	{
		/* slave */
		ApplySearchSlave(status->statusPtr, output, input, applyParams );
	}

	CHECKSTATUSPTR (status);

	DETATCHSTATUSPTR (status);
	RETURN(status);
}

/*
 * Function ApplySearchMaster
 *
 * Called from LALApplySearch only on the search master node. It's
 * main job is to wait for responses for each slave ie. retrieve the
 * statistics and wait for the slaves to end, then make the output.
 */

void ApplySearchMaster
(
	LALStatus		*status,
	LALSearchOutput		*output,
	LALSearchInput		*input,
	LALApplySearchParams	*applyParams
)
{
	/*  Variable Declaration  */
	BlockSearchParams	*myParams;
	REAL8Vector		*tempVector;
	MPIMessage		mpiMsg;
	MPI_Comm		*comm;
	UINT4			currSlaves;

	INITSTATUS( status, "ApplySearchMaster", BLOCKAPPLYSEARCHC );
	ATTATCHSTATUSPTR(status);

	/*  Variable Initialization  */
	comm		= ((MPI_Comm*)(applyParams->mpiParams->comm));
	myParams	= ((BlockSearchParams*)(applyParams->searchParams));
	tempVector	= NULL;
	currSlaves	= myParams->numSlaves;

	LALDCreateVector(status->statusPtr, &tempVector, myParams->numSlaves );
	CHECKSTATUSPTR( status );
	
	ASSERT (!output->result, status, 2, "result allocated");

	while(currSlaves > 0 )
	{
		LALMPIRecvMsg( status->statusPtr, &mpiMsg, (*comm));
		CHECKSTATUSPTR( status );

		LALMPIRecvREAL8( status->statusPtr, &(myParams->res->data[mpiMsg.source-1]), mpiMsg.source, (*comm));
		CHECKSTATUSPTR( status );

		tempVector->data[mpiMsg.source-1] = myParams->res->data[mpiMsg.source-1];

		currSlaves--;
	}

	BuildOutput( status->statusPtr, output, tempVector , myParams );

	/*  Clean up temporary Mem each time used  */
	LALDDestroyVector(status->statusPtr, &tempVector);
	CHECKSTATUSPTR( status );
	tempVector	= NULL;	

	output->fracRemaining	= 0;
	output->notFinished	= 0;

	DETATCHSTATUSPTR(status);
	RETURN(status);
}

void ApplySearchSlave
(
	LALStatus		*status,
	LALSearchOutput		*output,
	LALSearchInput		*input,
	LALApplySearchParams	*applyParams
)
{
	/*  Variable Declaration  */
	BlockSearchParams	*myParams;
	MPIMessage		sendMsg;
	MPI_Comm		*comm;
	REAL8			sum;
	INT4			iterator;

	INITSTATUS( status, "ApplySearchSlave", BLOCKAPPLYSEARCHC );
	ATTATCHSTATUSPTR(status);

	/*  Variable Initialization  */
	comm		= ((MPI_Comm*)(applyParams->mpiParams->comm));
	myParams	= ((BlockSearchParams*)(applyParams->searchParams));
	sendMsg.source	= myParams->rank;
	sendMsg.msg	= PassData;
	sendMsg.send	= 1;
	iterator	= 0;
	sum		= 0;

	LALDMoment(status->statusPtr, &sum, myParams->vec, myParams->maxMoment);

	/*  Send Message to say I am ready to send */
	LALMPISendMsg( status->statusPtr, &sendMsg, 0, (*comm));
	CHECKSTATUSPTR( status );	

	/*  Here comes the Result  */
	LALMPISendREAL8( status->statusPtr, &(sum), 0, (*comm));
	CHECKSTATUSPTR (status);
	
	output->numOutput	= 0;
	output->result		= NULL;
	output->fracRemaining	= 0;
	output->notFinished	= 0;

	RETURN(status);
}
