/******** <lalVerbatim file="FCTMyInterfCV"> ******** 
Author: Wen, L. 
$Id: FCTMyInterf.c,v 1.1 2001/12/13 00:32:28 teviet Exp $ 
********* </lalVerbatim> ********/ 
#include <FCTGeneral.h>
#include <FCTApplySearch.h>

NRCSID( FCTMYINTERFC, "$Id: FCTMyInterf.c,v 1.1 2001/12/13 00:32:28 teviet Exp $");

/* #define NUM_DATA_CUBES 1  */ /* don't know why it is here */

void
LALMyFCTInitPlan(LALStatus *status, FCTInStrucType *fctInStrucs, LALFCTPlan  **fctPlanPtr)
{
  
  

  LALFCTPlan * fctPlan=0;
  
  INITSTATUS( status, "FCTMYINTERFC", FCTMYINTERFC ); 
  ATTATCHSTATUSPTR (status);
  
  /* equivalent to fct_init_plan() and  fct_add_phase_function() */

   
  LALCreateFCTPlan(status->statusPtr,&fctPlan,&(fctInStrucs->planIn));  
  
  CHECKSTATUSPTR (status);   

  LALFCTSetUnits(status->statusPtr,&(fctInStrucs->setUnitsIn),fctPlan);
  CHECKSTATUSPTR (status);  


  LALFCTSetMaxSegments(status->statusPtr, fctInStrucs->maxFFTSeg, fctPlan);
  CHECKSTATUSPTR (status);   

#ifdef FCT_SO_DEBUG

  if(!fctPlan)
  {
    ABORT( status, 1, "null fctPlan1");
  }
#endif

  *fctPlanPtr=fctPlan;

  
  DETATCHSTATUSPTR (status);
  RETURN( status ); 

}


void 
LALMyFCTInterf(LALStatus *status,  FCTHierEventType **event, FCTSearchParams *mySearchParams)
{
 
  LALFCTPlan                    *fctPlan;
  UINT8                     outputDataSize=0;
  COMPLEX8Vector*            fctOut=0;
  LALFCTSetDataCubesInput   dataCubesIn;
  FCTInParamType                *fctInParams;

#ifdef FCT_SO_DEBUG      
  INT4                        j;
#endif  


  INITSTATUS( status, "LALMyFCTInterf", FCTMYINTERFC ); 
  ATTATCHSTATUSPTR (status);

  /*
    fprintf(stdout," in MyFctInterf ()\n");
    fflush( stdout );
  */
   
  fctInParams=mySearchParams->fctInParams;
  fctPlan=fctInParams->fctPlan;


  dataCubesIn.data_cube=fctInParams->cube;
  dataCubesIn.num_data_cubes =fctInParams->numDataCubes;

  /* temp debug */
#ifdef FCT_SO_DEBUG      

  printf("before dataCubeIn\n");
  for (j=0;j<fctInParams->dimension;j++)
  {
    printf("cube: start/end locations are %i,%i stride = %i\n", fctInParams->cube->start_locations[j],fctInParams->cube->end_locations[j], fctInParams->cube->stride[j]);
  }
#endif
    
  /*  call FCT search here and return event list */

  LALFCTSetDataCubes(status->statusPtr, &dataCubesIn,fctPlan);
  CHECKSTATUSPTR (status);   
  
  LALFCTOutputDataSize(status->statusPtr,&outputDataSize, fctPlan);
  CHECKSTATUSPTR (status);   
   
#ifdef FCT_SO_DEBUG_DATA

  for (j=0;j<fctInParams->fctInput->length;j++)
  {
    
    printf("fctInput= %g,%g\n",fctInParams->fctInput->data[j].re, fctInParams->fctInput->data[j].im); 
  }

#endif

  /*  Allocate an output vector of this size  */
  LALCCreateVector(status->statusPtr, &fctOut, outputDataSize);
  CHECKSTATUSPTR (status);

  LALFCTCalculate(status->statusPtr, fctOut, fctInParams->fctInput, fctPlan);
  CHECKSTATUSPTR (status);

#ifdef FCT_SO_DEBUG_DATA
  for (j=0;j<outputDataSize;j++)
  {
    printf("fctOut = %g,%g\n",fctOut->data[j].re, fctOut->data[j].im); 
  }
  
#endif

  /*  Check fctInParams->fctInput has to be Complex8  */
  LALFCTBuildEvent(status->statusPtr, fctOut, mySearchParams); /*  creat a linked eventlist */ 


  CHECKSTATUSPTR (status);

  /*  free allocated memories */

  LALCDestroyVector(status->statusPtr, &fctOut);
  CHECKSTATUSPTR (status);

  DETATCHSTATUSPTR (status);  
  RETURN( status ); 
}


void
LALFCTBuildEvent( LALStatus       *status,
		  COMPLEX8Vector  *fctOut,
		  FCTSearchParams *mySearchParams )
{
  INT4 i;          /* index over dimensions */
  INT4 dimLength;  /* length of each dimension */
  INT4 dimLength0; /* length of time dimension */
  INT4 j;          /* index over flattened data array */
  INT4 k;          /* index over event list */

  INITSTATUS( status, "LALFCTBuildEvent", FCTMYINTERFC ); 

  /* Check arguments. */
  ASSERT( fctOut, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( fctOut->data, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->fctInParams, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->fctInParams->freqMoment, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->fctInParams->cube, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->fctInParams->cube->start_locations, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->fctInParams->cube->end_locations, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->fctInParams->cube->stride, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->hierLevelUnits, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->hierLevelEvents, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1], status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->numHierLevelEvents, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->lowestEvent, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->powerThreshold, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].physicalStepSize, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
  ASSERT( mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].fctUnits, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );

  /* Check that the size of the output is what we expect. */
  dimLength = 1;
  for ( i = 0; i < mySearchParams->fctInParams->dimension; i++ ) {
#ifdef FCT_SO_DEBUG
    printf( "diff_location[%i] = %i, stride[%i] = %i\n", i, mySearchParams->fctInParams->cube->end_locations[i] - mySearchParams->fctInParams->cube->start_locations[i], i, mySearchParams->fctInParams->cube->stride[i] );
#endif
    dimLength *= (INT4)ceil( (REAL4)( mySearchParams->fctInParams->cube->end_locations[i] - mySearchParams->fctInParams->cube->start_locations[i] )
			     / (REAL4)( mySearchParams->fctInParams->cube->stride[i] ) );
  }
  dimLength0 = (INT4)ceil( (REAL4)( mySearchParams->fctInParams->cube->end_locations[0] - mySearchParams->fctInParams->cube->start_locations[0] )
			   / (REAL4)( mySearchParams->fctInParams->cube->stride[0] ) );
#ifdef FCT_SO_DEBUG
  printf( "dimLength = %i, fctOut->length = %i\n", dimLength, fctOut->length );
#endif
  ASSERT( dimLength == fctOut->length, status, APPLYSEARCH_EDIM, APPLYSEARCH_MSGEDIM );

  /* Look for events above threshold. */
  for ( j = 0; j < fctOut->length; j++ ) {
    REAL4 power = ( fctOut->data[j].re )*( fctOut->data[j].re )/mySearchParams->fctInParams->freqMoment->sigma2.re;
    power += ( fctOut->data[j].im )*( fctOut->data[j].im )/mySearchParams->fctInParams->freqMoment->sigma2.im;
    power /= mySearchParams->fctInParams->dataLength/2.0;
    if ( power > mySearchParams->powerThreshold[mySearchParams->curHierLevel] ) {
      /* Require event to be in the middle half of the input data. */
      REAL4 tc = mySearchParams->fctInParams->cube->start_locations[0] + ( j % dimLength0 )*mySearchParams->fctInParams->cube->stride[0];
      tc *= mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].fctUnits[0];
      if ( mySearchParams->curHierLevel == 0 ) {
	tc /= mySearchParams->inputDataStepSize*mySearchParams->inputDataLength;
	tc -= 0.5;
      } else {
	tc -= mySearchParams->curEvent->location[0];
	tc /= mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].range[0];
      }
      if ( tc > -0.25 && tc < 0.25 ) {
	INT4 jTmp = j;              /* temporary index */
	FCTHierEventType *curEvent; /* event we are currently dealing with */
	FCTHierEventType *oldEvent; /* old lowest event */

	/* Swap out old lowest event. */
	oldEvent = mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1];
	if ( mySearchParams->lowestEvent[mySearchParams->curHierLevel+1] == 0 ) {
	  curEvent = createHierEvent( mySearchParams->fctInParams->dimension );
	  if ( curEvent )
	    mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1] = curEvent;
	} else {
	  for ( k = 0; k < mySearchParams->lowestEvent[mySearchParams->curHierLevel+1]; k++ ) {
	    curEvent = oldEvent;
	    oldEvent = oldEvent->nextEvent;
	    ASSERT( oldEvent, status, APPLYSEARCH_ENULL, APPLYSEARCH_MSGENULL );
	  }
	  curEvent->nextEvent = createHierEvent( mySearchParams->fctInParams->dimension );
	  curEvent = curEvent->nextEvent;
	}
	if ( !curEvent ) {
	  ABORT( status, APPLYSEARCH_EALOC, APPLYSEARCH_MSGEALOC );
	}
	curEvent->nextEvent = oldEvent->nextEvent;
	LALFree( oldEvent->location );
	LALFree( oldEvent );

	/* Compute data for new event. */
	for ( i = 0; i < mySearchParams->fctInParams->dimension; i++ ) {
	  dimLength = (INT4)ceil( (REAL4)( mySearchParams->fctInParams->cube->end_locations[i] - mySearchParams->fctInParams->cube->start_locations[i] )
				  / (REAL4)( mySearchParams->fctInParams->cube->stride[i] ) );
	  curEvent->location[i] = mySearchParams->fctInParams->cube->start_locations[i]
	    + ( jTmp % dimLength )*mySearchParams->fctInParams->cube->stride[i];
	  curEvent->location[i] *= mySearchParams->hierLevelUnits[mySearchParams->curHierLevel].fctUnits[i];
	  jTmp /= dimLength;
	}
	curEvent->power = power;

	/* Determine new lowest event and threshold. */
	for ( k = 0, curEvent = mySearchParams->hierLevelEvents[mySearchParams->curHierLevel+1];
	      curEvent; k++, curEvent = curEvent->nextEvent )
	  if ( curEvent->power < power ) {
	    mySearchParams->lowestEvent[mySearchParams->curHierLevel+1] = k;
	    power = curEvent->power;
	  }
	mySearchParams->powerThreshold[mySearchParams->curHierLevel] = power;

	/* Done adding event. */
      }
    }
  }
  RETURN( status );
}


void
LALFCTBuildEventBak(LALStatus *status, COMPLEX8Vector *fctOut, FCTHierEventType  **event, FCTSearchParams *mySearchParams)
{
  FCTHierEventType *myEvent;
  FCTInParamType   *fctInParams;

  INT4   i,j,dimension,dimLength;
  UINT4  length;

  REAL8  power=0.;
  REAL8  avgP=0.;
  REAL8  sigmaP=0.;
  REAL8  maxP=0., maxProb=1,localMaxP=0,localMaxProb=1;
  REAL8  prob=1; /* false alarm Prob */

  INT4   maxJJ=0,tCnt=0,localMaxJJ=0;
  INT4   *maxJ=0,*maxJJJ=0;

  /* check hard wired */
  INT4 const numJ=5;
  INT4 const numD=1;

  /* debug */
  /*          printf(" in BuildEvent\n"); */

  fctInParams=mySearchParams->fctInParams;
  dimension=fctInParams->dimension;
  maxJ=(INT4 *) LALMalloc(sizeof(INT4)*numJ); /*  5 specific to inspiral Check */
  maxJJJ=(INT4 *) LALMalloc(sizeof(INT4)*numD); /* maximum number of slides */
  /*   myEvent=NULL; */

  length=fctOut->length/numD;

  /* initialize the event list */
  /* allocate here, deallocate outside or at the end if #event =0 */

  fctInParams->numEvents=0;

  for (i=0;i<numJ;i++) {
    maxJ[i]=0;
  }

  /* use a sliding window */
  /* calculate power (Power=2*Power/N/sigma2), prob, avgP, sigmaP first, power stores in fctOut */

  /* debug */
  /* printf(" in BuildEvent 1\n"); */

  for (i=0;i<numD;i++) {
    localMaxP=0.;
    localMaxProb=1.;
    localMaxJJ=0;
    for (j=i*length;j<(i+1)*length;j++) {

      /* debug */
      /*      printf(" in BuildEvent 2\n");*/

      power=((fctOut->data[j].re) *(fctOut->data[j].re)/fctInParams->freqMoment->sigma2.re
	     +(fctOut->data[j].im)*(fctOut->data[j].im)/fctInParams->freqMoment->sigma2.im)
	/fctInParams->dataLength/2.;
      /* debug */
      /*      printf(" in BuildEvent 3\n");*/

#ifdef FCT_SO_DEBUG_DATA
      printf("power=%f, sisgma=%e,%e\n",power,fctInParams->freqMoment->sigma2.re,fctInParams->freqMoment->sigma2.im);
#endif

      prob=1.-pow((1.-exp(-power)),fctInParams->dataLength);
      if (power >localMaxP) {
	localMaxP=power;
	localMaxProb=prob;
	localMaxJJ=j;
      }
      avgP +=power;
      sigmaP +=power*power;

    }/* end for j*/

    /* debug */
    /*     printf(" in BuildEvent 4\n"); */

    if (localMaxP>maxP) {
      maxP=localMaxP;
      maxProb=localMaxProb;
      maxJJ=localMaxJJ;
    }

    if (localMaxProb<=fctInParams->prob) {
      fctOut->data[tCnt].re=localMaxP;
      fctOut->data[tCnt].im=localMaxProb;
      maxJJJ[tCnt]=localMaxJJ;
      tCnt++;
#ifdef FCT_SO_DEBUG
      printf(" maxP, maxProb, maxJJ=%e %e,%i\n",localMaxP,localMaxProb,localMaxJJ);
#endif

    }
  }/*end for i */

  length=length*numD;
  avgP/=length; 
  sigmaP/=(length-1);
  sigmaP -=avgP*avgP/(length-1)*length;
  
#ifdef FCT_SO_DEBUG
  printf("debug\n");
  for (i=0;i<dimension;i++) {
    printf("start,stop,stride=%i,%i,%i\n",fctInParams->cube->start_locations[i],fctInParams->cube->end_locations[i ],fctInParams->cube->stride[i]);
    dimLength=(fctInParams->cube->end_locations[i]-fctInParams->cube->start_locations[i ])/fctInParams->cube->stride[i];
    printf("debug2,maxJJ,dimLength=%i,%i\n",maxJJ,dimLength);
    if (dimLength !=0) {
      maxJ[i]=fctInParams->cube->start_locations[i ]+(maxJJ % dimLength)*fctInParams->cube->stride[i];
      maxJJ=maxJJ/dimLength;
    } else {
      printf("sth is wrong w/ the definition of start/end, or stride\n");
      maxJ[i]=fctInParams->cube->end_locations[i ];
    }

  }/* end for */

  printf(" in LALFCTBuildEvent()\n");
  printf(" length=%i,avgP=%e, sigP=%e, maxP=%e\n",length,avgP,sigmaP,maxP);
  printf(" max Power occurs at maxJJ=%i,%i,%i,%i,%i\n",maxJ[0],maxJ[1],maxJ[2],maxJ[3],maxJ[4]);
#endif

  /* make event list  for
     output top tCnt events  above threshold
  */

  /* debug */
  /*      printf(" in BuildEvent 3\n"); */

  if (tCnt !=0) {


    myEvent= *event=createHierEvent(mySearchParams->fctInParams->dimension);


    for (j=0;j<tCnt;j++) {
      power=fctOut->data[j].re;
      prob=fctOut->data[j].im;
      maxJJ=maxJJJ[j];
      for (i=0;i<dimension;i++) {
	maxJ[i]=0;
	dimLength=(fctInParams->cube->end_locations[i]-fctInParams->cube->start_locations[i ])/
	  fctInParams->cube->stride[i];
	maxJ[i]=fctInParams->cube->start_locations[i ]+(maxJJ % dimLength)*fctInParams->cube->stride[i];
	maxJJ=maxJJ/dimLength;
      }/* end for i */

#ifdef FCT_SO_DEBUG
      printf(" P above threshold at  maxJJ=%i,%i,%i,%i,%i\n",
	     maxJ[0],maxJ[1],maxJ[2],maxJ[3],maxJ[4]);
      printf(" P=%e, prob=%e\n",power,prob);
#endif

      myEvent->location[0]=maxJ[0]*mySearchParams->hierLevelUnits[j].physicalStepSize[0]; /* 1/Nf, check should input the time series start-time  */
      myEvent->location[1]=maxJ[1]*mySearchParams->hierLevelUnits[j].fctUnits[1];
      myEvent->location[2]=maxJ[2]*mySearchParams->hierLevelUnits[j].fctUnits[3];  /* 1228.81374; */
      myEvent->location[3]=maxJ[3]*mySearchParams->hierLevelUnits[j].fctUnits[2];
      myEvent->location[4]=maxJ[4]*mySearchParams->hierLevelUnits[j].fctUnits[4];
      myEvent->power=power;
      myEvent->nextEvent=createHierEvent(mySearchParams->fctInParams->dimension);
      myEvent=myEvent->nextEvent;
      myEvent->nextEvent=NULL; 
      fctInParams->numEvents++;  	    
    } /* end for j*/
  }/* endif tCnt!=0 */
  LALFree(maxJ);
  LALFree(maxJJJ);
}

/* calculate average, variance and anything else */
void
LALFCTMoment(LALStatus *status, INT4 length, COMPLEX8 *dataIn, FCTMomentType *moment)
{

 COMPLEX8  avgP;
 COMPLEX8  sigmaP;
 INT4   j;

 INITSTATUS( status, "LALFCTMoment", FCTMYINTERFC ); 
 ATTATCHSTATUSPTR (status);

 avgP.re=0.0;
 avgP.im=0.0;
 sigmaP.re=0.0;
 sigmaP.im=0.0;


 for (j=0;j<length;j++)
 {

   avgP.re +=dataIn[j].re;
   sigmaP.re +=(dataIn[j].re* dataIn[j].re);
   avgP.im +=dataIn[j].im;
   sigmaP.im +=(dataIn[j].im* dataIn[j].im);

#ifdef FCT_SO_DEBUG_DATA   
      if (fabs(avgP.re) > 1.0e54 || fabs(avgP.re) <1.0e-54)  
   {
     printf("j,dataIn,avgP.re,sigmaP=%i,%e,%e,%e\n",j,dataIn[j].re,avgP.re,sigmaP.re);
   }
#endif
 } /* end for */

 avgP.re/=length; 
 sigmaP.re/=(length-1);

/* debug */
 
 /*  printf("1. avgP.re, sigma.re= %e,%e\n",avgP.re,sigmaP.re); */
 
 sigmaP.re -=((avgP.re*avgP.re)/(length-1.)*length);
 avgP.im/=length; 
 

 sigmaP.im/=(length-1);
 sigmaP.im -=((avgP.im*length*avgP.im)/(length-1.));

 moment->avg=avgP;
 moment->sigma2=sigmaP;

 DETATCHSTATUSPTR (status);
 RETURN( status ); 
}
