/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="SlopeLALConditionDataC"> ********
Author: Daw, E
$Id: SlopeLALConditionData.c,v 1.1 2002/05/08 14:27:16 edaw Exp $
********* </lalVerbatim> ********/

#ifdef DEBUG_SLOPE
#include <stdio.h>
#endif
#include <math.h>

/* functions for memory allocation */
#include <lal/LALStdlib.h>

/* functions for vector creation and manipulation */
/* #include <lal/AVFactories.h> */

/* functions for LAL MPI interface */
#include <lal/Comm.h>

/* functions for slope filtering and peak searching data */
#include <lal/SlopeDetectorFilter.h>

/* structures and definitions for this wrapper code */
#include <SlopeDatatypes.h>

NRCSID( SLOPELALCONDITIONDATAC, "$Id: SlopeLALConditionData.c,v 1.1 2002/05/08 14:27:16 edaw Exp $");

/******** local static function declarations ********/

/****************************************************/
/** note that currently my code is designed to run **/
/** on a single machine. ****************************/
/****************************************************/

/******** wrapperAPI function definitions */

/* look in the data for the right channel, extract the */
/* channel data and any history buffer data to a vector */
/* of type REAL4. Get the start and end times of the data */
/* segment. */

void
LALConditionData(LALStatus*             status,
		 LALSearchInput*        inout,
		 void*                  searchParams,
		 LALMPIParams*          mpiParams)
{
  BOOLEAN dataFound = 0;
  UINT4 dCount = 0;
  UINT4 dElCount = 0;
  SlopeSearchParams* params = 0x0;
  UINT4 dataArraySize = 0;
  REAL8 historyBufferTimeInterval;
  REAL8 historyLengthSecs = 0;
  REAL8 historyLengthNSecs = 0;
#ifdef SLDEBUG
  UINT4 i;
#endif  

  INITSTATUS( status, "SlopeLALConditionData", SLOPELALCONDITIONDATAC );
  ATTATCHSTATUSPTR(status);
  
  params = (SlopeSearchParams*) searchParams;
  
  /*loop through the data sets in the input object and*/
  /*get the data from the channel I requested*/

  for(dCount = 0; dCount < inout->numberSequences; ++dCount) {

#ifdef SLDEBUG
    printf("%s\n",inout->sequences[dCount].name);
#endif

    if( strstr(inout->sequences[dCount].name,params->channelName) != NULL ) {
      
      /* error trap for non timeseries data here */
      if( inout->sequences[dCount].space != timeD ) {
	ABORT( status, CONDITIONDATA_NONTSDATA, CONDITIONDATA_NONTSDATAMESG);
      }
      
      dataFound = 1;
      
      /* set size of input data array including history if one exists */
      if( params->filtHistAllocated == 0 ) {
	dataArraySize = inout->sequences[dCount].dimensions[0];
      } else if( params->filtHistAllocated == 1) {
	dataArraySize = inout->sequences[dCount].dimensions[0] +
	  params->nSampInFiltHistBuf;
      } else {
	ABORT( status, CONDITIONDATA_INVALIDFILTHIST,
	       CONDITIONDATA_INVALIDFILTHISTMESG );
      }

#ifdef SLDEBUG
      printf("Size of data array is %u\n",dataArraySize);
      printf("Memory to be allocated is %u bytes.\n",dataArraySize *
	     sizeof(REAL4));
#endif

      /* allocate memory for input data */
      params->dataNew = LALCalloc( dataArraySize , sizeof(REAL4) );
      
      if( params->filtHistAllocated == 1 ) {
	/* set first elements of data vector to history buffer */
	for( dElCount = 0 ; dElCount < (params->nSampInFiltHistBuf) ;
	     ++dElCount ) {
	  params->dataNew[dElCount] = params->filtHistBuf[dElCount];
	}
      }

      /* read new data to rest of data vector. The */
      /* cast to reals depends on the original data type. For now */
      /* I support only REAL4, INT4S and INT2S */
      switch ( inout->sequences[dCount].type ) {
      case int_2s:
	if( params->filtHistAllocated == 1) {
	  for( dElCount = params->nSampInFiltHistBuf ; dElCount <
	       dataArraySize ; ++dElCount ) { 
	    params->dataNew[dElCount] = 
	      (REAL4) inout->sequences[dCount].data.int2s[dElCount 
		- params->nSampInFiltHistBuf ] ;
	  } 
	} else if (params->filtHistAllocated == 0) { 	  
	  for( dElCount = 0 ; dElCount < dataArraySize ; ++dElCount ) { 
	    params->dataNew[dElCount] = 
	      (REAL4) inout->sequences[dCount].data.int2s[dElCount] ;
	  } 
	} else {
	  ABORT( status, CONDITIONDATA_INVALIDFILTHIST,
		 CONDITIONDATA_INVALIDFILTHISTMESG );
	}
	break;
      case int_4s:
	if( params->filtHistAllocated == 1) {
	  for( dElCount = params->nSampInFiltHistBuf ; dElCount <
	       dataArraySize ; ++dElCount ) { 
	    params->dataNew[dElCount] = 
	      (REAL4) inout->sequences[dCount].data.int4s[dElCount 
		- params->nSampInFiltHistBuf ] ;
	  } 
	} else if (params->filtHistAllocated == 0) { 	  
	  for( dElCount = 0 ; dElCount < dataArraySize ; ++dElCount ) { 
	    params->dataNew[dElCount] = 
	      (REAL4) inout->sequences[dCount].data.int4s[dElCount] ;
	  } 
	} else {
	  ABORT( status, CONDITIONDATA_INVALIDFILTHIST,
		 CONDITIONDATA_INVALIDFILTHISTMESG );
	}
	break;
      case real_4:
	if( params->filtHistAllocated == 1) {
	  for( dElCount = params->nSampInFiltHistBuf ; dElCount <
	       dataArraySize ; ++dElCount ) { 
	    params->dataNew[dElCount] = 
	      inout->sequences[dCount].data.real4[dElCount 
		- params->nSampInFiltHistBuf ] ;
	  } 
	} else if (params->filtHistAllocated == 0) { 	  
	  for( dElCount = 0 ; dElCount < dataArraySize ; ++dElCount ) { 
	    params->dataNew[dElCount] = 
	      inout->sequences[dCount].data.real4[dElCount] ;
	  } 
	} else {
	  ABORT( status, CONDITIONDATA_INVALIDFILTHIST,
		 CONDITIONDATA_INVALIDFILTHISTMESG );
	}
	break;
      default:
	ABORT( status, CONDITIONDATA_DATATYPEUNSUP,
	       CONDITIONDATA_DATATYPEUNSUPMESG );
      }

      /* set data array length */
      params->nData = dataArraySize;

      if( params->nSampInFiltHistBuf > 0 ) {

	/* get duration of history buffer */
	historyBufferTimeInterval = (REAL8)params->nSampInFiltHistBuf /
	  (REAL8)params->samplingRate;

	/* split history duration into secs, nsecs. */
	historyLengthNSecs = 1e+9 * modf((REAL8)historyBufferTimeInterval,
				&historyLengthSecs);

	/* get start time of history buffer assuming it is contiguous */
	/* with the current data segment */

	if( (UINT4)historyLengthSecs > 
	    inout->sequences[dCount].range.dTime.startSec ) {
	  ABORT( status, CONDITIONDATA_TIMEARITHERROR,
		 CONDITIONDATA_TIMEARITHERRORMESG );
	} else {
	  if ( (UINT4)historyLengthNSecs <= 
	       inout->sequences[dCount].range.dTime.startNan ) {
	    /* subtract without carry */
	    params->startTimeSec = 
	      inout->sequences[dCount].range.dTime.startSec -
	      (UINT4)historyLengthSecs;
	    params->startTimeNan =
	      inout->sequences[dCount].range.dTime.startNan -
	      (UINT4)historyLengthNSecs;
	  }
	  else {
	    /* subtract with carry */
	    params->startTimeSec = 
	      inout->sequences[dCount].range.dTime.startSec -
	      (UINT4)historyLengthSecs - 1;
	    params->startTimeNan =
	      (UINT4)1000000000 + 
	      inout->sequences[dCount].range.dTime.startNan -
	      (UINT4)historyLengthNSecs;
	  }
	}
      } else {
	params->startTimeSec = 
	  inout->sequences[dCount].range.dTime.startSec -
	  (UINT4)historyLengthSecs - 1;
	params->startTimeNan =
	  (UINT4)1000000000 + 
	  inout->sequences[dCount].range.dTime.startNan -
	  (UINT4)historyLengthNSecs;
      }
    
      /* break out of for loop once data located */
      if(dataFound == 1) {
	break;
      }
      
    }
    
    if( dataFound == 0 ) {
      /* error and exit, data channel was not present */
      ABORT( status, CONDITIONDATA_NOSUCHCHANNEL,
	     CONDITIONDATA_NOSUCHCHANNELMESG );
    }

  }

#ifdef SLDEBUG
  printf("OK, reached end of condition data.\n");
#endif

  CHECKSTATUSPTR(status);
  DETATCHSTATUSPTR(status);
  RETURN(status);
}
