/**** <lalVerbatim file="BuildDBCV">
 * $Id: BuildDB.c,v 1.2 2002/06/05 20:19:24 jolien Exp $
 **** </lalVerbatim> */
/**** <lalLaTeX>
 *
 * \subsection{Module \texttt{BuildDB.c}}
 * \label{ss:BuildDB.c}
 *
 * This module provides the routines used to create \texttt{dataBase}
 * entries required as output from \texttt{LALApplySearch()}.
 *
 * \subsubsection*{Prototypes}
 * \input{BuildDBCP}
 *
 * \subsubsection*{Description}
 *
 * The routine \texttt{LALCreateBLOB()} copies a Binary Large OBject (BLOB),
 * given as input along with its size in bytes, to a \texttt{LALBLOB} structure,
 * which is created.  The routine \texttt{LALDestroyBLOB()} destroys
 * a \texttt{LALBLOB} structure.
 *
 * The routine \texttt{LALCreateNextDBEntry()} allocates memory for a new
 * \texttt{dataBase} structure, a pointer to this structure is returned as
 * the output.  If the output is not a pointer to \texttt{NULL}, the new
 * \texttt{dataBase} is attached to the existing linked list (the pointer
 * to the next structure must be \texttt{NULL}).  As input this routine
 * requires the table name and the column name for the database entry.
 * 
 * The routine \texttt{LALCreateDBEntryData()} allocates memory for the
 * data to be contained in the \texttt{dataBase} structure.  This routine
 * takes as input the type and number of rows of data that will be put into the
 * database entry column.  After the routine is called, the calling routine
 * must populate the data array allocated by this routine.  If a Binary Large
 * OBject (BLOB) is to be entered into the database, use
 * \texttt{LALCreateDBEntryBLOB()} instead.  This routine creates a number of
 * copies of the BLOB as there are rows in the column.  (There is no routine
 * to put different BLOBs in the different rows; this must be done by hand.)
 * 
 * The routine \texttt{LALCreateSearchSummaryDataBase()} is a higher-level
 * routine that creates and populates a \texttt{dataBase} structure containing
 * all the information required for a \texttt{search\_summary} database entry.
 *
 * There are no routines to destroy the \texttt{dataBase} structures created
 * by these routines.  This is done automatically by the
 * \texttt{LALWrapperInterface} after the \texttt{wrapperAPI} has processed
 * them.
 *
 * \subsubsection*{Example}
 *
 * Building output in \texttt{LALApplySearch()} can be somewhat cumbersome.
 * Here one example of a output building routine.  It creates two output
 * results, one is the \texttt{search\_summary} database entry, the second
 * is a \texttt{sgnl\_unmodeled} database entry.  The latter consists of a
 * number of columns, each with a number of rows equal to the number of events
 * (each event is a row in the \texttt{sgnl\_unmodeled} table).
 *
 **** </lalLaTeX> */
#if 0
<lalLaTeX>
\begin{verbatim}
typedef struct tagSillyEvent
{
  LIGOTimeGPS  epoch;
  REAL4        duration;
  REAL4        amplitude;
  REAL4        snr;
  REAL4        confidence;
}
SillyEvent;

typedef struct tagSillyResults
{
  CHAR         ifoName[3];
  CHAR         channel[64];
  LIGOTimeGPS  inputStartTime;
  LIGOTimeGPS  inputEndTime;
  LIGOTimeGPS  outputStartTime;
  LIGOTimeGPS  outputEndTime;
  UINT4        numNodes;
  UINT4        numEvents;
  SillyEvent  *events;
}
SillyResults;

void
SillyBuildOutput(
    LALStatus       *status,
    LALSearchOutput *output,
    SillyResults    *input
    )
{
  CHAR                  searchName[] = "silly";
  LALBLOB               blob;
  SearchSummaryInfo     info;
  dataBase             *dbase;
  UINT4                 event;

  INITSTATUS( status, "SillyBuildOutput", "Silly: $Id: BuildDB.c,v 1.2 2002/06/05 20:19:24 jolien Exp $" );
  ATTATCHSTATUSPTR( status );

  output->numOutput = 2;
  output->results = LALCalloc( output->numOutput, sizeof( *output->results ) );
  if ( ! output->results )
  {
    ABORT( status, 1, "Allocation error" );
  }
  output->result[0].search = burst;
  output->result[1].search = burst;
  output->result[0].significant = 1;
  output->result[1].significant = 1;

  /* first output: the search_summary database entry */
  dbase = NULL;
  info.dsoName         = searchName;
  info.inputStartTime  = input->inputStartTime;
  info.inputEndTime    = input->inputEndTime;
  info.outputStartTime = input->outputStartTime;
  info.outputEndTime   = input->outputEndTime;
  info.numEvents       = input->numEvents;
  info.numNodes        = input->numNodes;
  LALCreateSearchSummaryDataBase( status->statusPtr, &dbase, &info );
  CHECKSTATUSPTR( status );
  output->result[0].results = dbase;

  /* second output: the sgnl_unmodeled database entry with numEvent rows */
  dbase = NULL;

  /* column "search" */
  LALCreateNextDBEntry( status->statusPtr, &dbase, "sngl_unmodeled", "search" );
  CHECKSTATUSPTR( status );
  output->results[1].results = dbase; /* bind head of list to the output */
  blob.data = search;
  blob.size = strlen( search );
  LALCreateDBEntryBLOB( status->statusPtr, dbase, &blob, input->numEvents );
  CHECKSTATUSPTR( status );

  /* column "ifo" */
  LALCreateNextDBEntry( status->statusPtr, &dbase, "sngl_unmodeled", "ifo" );
  CHECKSTATUSPTR( status );
  blob.data = input->ifoName;
  blob.size = 2;
  LALCreateDBEntryBLOB( status->statusPtr, dbase, &blob, input->numEvents );
  CHECKSTATUSPTR( status );
  
  /* column "channel" */
  LALCreateNextDBEntry( status->statusPtr, &dbase, "sngl_unmodeled", "channel" );
  CHECKSTATUSPTR( status );
  blob.data = input->channel;
  blob.size = strlen( input->channel );
  LALCreateDBEntryBLOB( status->statusPtr, dbase, &blob, input->numEvents );
  CHECKSTATUSPTR( status );

  /* column "start_time" */
  LALCreateNextDBEntry( status->statusPtr, &dbase, "sngl_unmodeled", "start_time" );
  CHECKSTATUSPTR( status );
  LALCreateDBEntryBLOB( status->statusPtr, dbase, int_4s, input->numEvents );
  CHECKSTATUSPTR( status );
  for ( event = 0; event < input->numEvents; ++event )
  {
    dbase->rows.int4s[event] = input->events[event].epoch.gpsSeconds;
  }

  /* column "start_time_ns" */
  LALCreateNextDBEntry( status->statusPtr, &dbase, "sngl_unmodeled", "start_time_ns" );
  CHECKSTATUSPTR( status );
  LALCreateDBEntryBLOB( status->statusPtr, dbase, int_4s, input->numEvents );
  CHECKSTATUSPTR( status );
  for ( event = 0; event < input->numEvents; ++event )
  {
    dbase->rows.int4s[event] = input->events[event].epoch.gpsNanoSeconds;
  }

  /* column "duration" */
  LALCreateNextDBEntry( status->statusPtr, &dbase, "sngl_unmodeled", "duration" );
  CHECKSTATUSPTR( status );
  LALCreateDBEntryBLOB( status->statusPtr, dbase, real_4, input->numEvents );
  CHECKSTATUSPTR( status );
  for ( event = 0; event < input->numEvents; ++event )
  {
    dbase->rows.real4[event] = input->events[event].duration;
  }

  /* column "amplitude" */
  LALCreateNextDBEntry( status->statusPtr, &dbase, "sngl_unmodeled", "amplitude" );
  CHECKSTATUSPTR( status );
  LALCreateDBEntryBLOB( status->statusPtr, dbase, real_4, input->numEvents );
  CHECKSTATUSPTR( status );
  for ( event = 0; event < input->numEvents; ++event )
  {
    dbase->rows.real4[event] = input->events[event].amplitude;
  }

  /* column "snr" */
  LALCreateNextDBEntry( status->statusPtr, &dbase, "sngl_unmodeled", "snr" );
  CHECKSTATUSPTR( status );
  LALCreateDBEntryBLOB( status->statusPtr, dbase, real_4, input->numEvents );
  CHECKSTATUSPTR( status );
  for ( event = 0; event < input->numEvents; ++event )
  {
    dbase->rows.real4[event] = input->events[event].snr;
  }
  
  /* column "confidence" */
  LALCreateNextDBEntry( status->statusPtr, &dbase, "sngl_unmodeled", "confidence" );
  CHECKSTATUSPTR( status );
  LALCreateDBEntryBLOB( status->statusPtr, dbase, real_4, input->numEvents );
  CHECKSTATUSPTR( status );
  for ( event = 0; event < input->numEvents; ++event )
  {
    dbase->rows.real4[event] = input->events[event].snr;
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}
\end{verbatim}
</lalLaTeX>
#endif /*** comment ***/
/**** <lalLaTeX>
 *
 * \vfill{\footnotesize\input{BuildDBCV}}
 *
 **** </lalLaTeX> */

#include <lal/LALStdlib.h>
#include <LALWrapperConfig.h>
#include <BuildDB.h>

NRCSID( BUILDDBC, "$Id: BuildDB.c,v 1.2 2002/06/05 20:19:24 jolien Exp $" );

extern const char *lalCVSTag;
static const char *lalWrapperCVSTag = LAL_WRAPPER_CVS_TAG;

static size_t sizeof_datatype( datatype type )
{
  switch ( type )
  {
    case boolean_lu:
      return sizeof( BOOLEAN );
    case char_s:
      return sizeof( CHAR );
    case char_u:
      return sizeof( UCHAR );
    case int_2s:
      return sizeof( INT2 );
    case int_2u:
      return sizeof( UINT2 );
    case int_4s:
      return sizeof( INT4 );
    case int_4u:
      return sizeof( UINT4 );
    case int_8s:
      return sizeof( INT8 );
    case int_8u:
      return sizeof( UINT8 );
    case real_4:
      return sizeof( REAL4 );
    case real_8:
      return sizeof( REAL8 );
    case complex_8:
      return sizeof( COMPLEX8 );
    case complex_16:
      return sizeof( COMPLEX16 );
    default:
      return 0;
  }
  return 0;
}

/* <lalVerbatim file="BuildDBCP"> */
void
LALCreateBLOB(
    LALStatus   *status,
    LALBLOB    **output,
    const void  *input,
    UINT4        size
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALCreateBLOB", BUILDDBC );
  ASSERT( output, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( ! *output, status, BUILDDBH_ENNUL, BUILDDBH_MSGENNUL );
  ASSERT( size, status, BUILDDBH_ESIZE, BUILDDBH_MSGESIZE );
  *output = LALCalloc( 1, sizeof( **output ) );
  if ( ! *output )
  {
    ABORT( status, BUILDDBH_EALOC, BUILDDBH_MSGEALOC );
  }
  (*output)->size = size;
  (*output)->data = LALMalloc( (*output)->size );
  if ( ! (*output)->data )
  {
    LALFree( *output );
    *output = NULL;
    ABORT( status, BUILDDBH_EALOC, BUILDDBH_MSGEALOC );
  }
  if ( input )
  {
    memcpy( (*output)->data, input, (*output)->size );
  }
  else
  {
    memset( (*output)->data, 0, (*output)->size );
  }
  RETURN( status );
}

/* <lalVerbatim file="BuildDBCP"> */
void
LALDestroyBLOB(
    LALStatus  *status,
    LALBLOB   **blob
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALDestroyBLOB", BUILDDBC );
  ASSERT( blob, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( *blob, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( (*blob)->data, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  LALFree( (*blob)->data );
  LALFree( *blob );
  *blob = NULL;
  RETURN( status );
}

/* <lalVerbatim file="BuildDBCP"> */
void
LALCreateNextDBEntry( 
    LALStatus   *status,
    dataBase   **output,
    const CHAR  *table,
    const CHAR  *column
    )
{ /* </lalVerbatim> */
  dataBase *dbEntry;
  INITSTATUS( status, "LALCreateNextDBEntry", BUILDDBC );
  ASSERT( table, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( column, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( output, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( ! *output || ! (*output)->next, status,
      BUILDDBH_ENNUL, BUILDDBH_MSGENNUL );
  dbEntry = LALCalloc( 1, sizeof( *dbEntry ) );
  strncpy( dbEntry->tableName, table, sizeof( dbEntry->tableName ) );
  strncpy( dbEntry->columnName, column, sizeof( dbEntry->columnName ) );
  if ( *output )
    (*output)->next = dbEntry;
  *output = dbEntry;
  RETURN( status );
}

/* <lalVerbatim file="BuildDBCP"> */
void
LALCreateDBEntryData( 
    LALStatus *status,
    dataBase  *output,
    datatype   type,
    UINT4      rows
    )
{ /* </lalVerbatim> */
  size_t size;
  INITSTATUS( status, "LALCreateDBEntryData", BUILDDBC );
  ASSERT( output, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( rows, status, BUILDDBH_EROWS, BUILDDBH_MSGEROWS );
  size = sizeof_datatype( type );
  ASSERT( size, status, BUILDDBH_ETYPE, BUILDDBH_MSGETYPE );
  output->type          = type;
  output->numberRows    = rows;
  output->rowDimensions = NULL;
  output->rows.chars    = LALCalloc( rows, size );
  if ( ! output->rows.chars )
  {
    ABORT( status, BUILDDBH_EALOC, BUILDDBH_MSGEALOC );
  }
  RETURN( status );
}

/* <lalVerbatim file="BuildDBCP"> */
void
LALCreateDBEntryBLOB(
    LALStatus *status,
    dataBase  *output,
    LALBLOB   *blob,
    UINT4      rows
    )
{ /* </lalVerbatim> */
  UINT4 row;
  INITSTATUS( status, "LALCreateDBEntryBLOB", BUILDDBC );
  ASSERT( output, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( rows, status, BUILDDBH_EROWS, BUILDDBH_MSGEROWS );
  ASSERT( blob, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( blob->data, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( blob->size, status, BUILDDBH_ESIZE, BUILDDBH_MSGESIZE );
  output->type          = char_s_ptr;
  output->numberRows    = rows;
  output->rowDimensions = LALCalloc( rows, sizeof( *output->rowDimensions ) );
  if ( ! output->rowDimensions )
  {
    ABORT( status, BUILDDBH_EALOC, BUILDDBH_MSGEALOC );
  }
  output->rows.chars = LALCalloc( rows, blob->size );
  if ( ! output->rows.chars )
  {
    LALFree( output->rowDimensions );
    output->rowDimensions = NULL;
    ABORT( status, BUILDDBH_EALOC, BUILDDBH_MSGEALOC );
  }
  for ( row = 0; row < rows; ++row )
  {
    output->rowDimensions[row] = blob->size;
    memcpy( output->rows.chars + row * blob->size, blob->data, blob->size );
  }
  RETURN( status );
}

/* <lalVerbatim file="BuildDBCP"> */
void
LALCreateSearchSummaryDataBase(
    LALStatus          *status,
    dataBase          **output,
    SearchSummaryInfo  *info
    )
{ /* </lalVerbatim> */
  LALBLOB  *blob = NULL;
  dataBase *dbase;

  INITSTATUS( status, "LALCreateSearchSummaryDataBase", BUILDDBC );
  ATTATCHSTATUSPTR( status );

  ASSERT( output, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );
  ASSERT( ! *output, status, BUILDDBH_ENNUL, BUILDDBH_MSGENNUL );
  ASSERT( info, status, BUILDDBH_ENULL, BUILDDBH_MSGENULL );

  TRY( LALCreateBLOB( status->statusPtr, &blob, info->dsoName,
        strlen( info->dsoName ) ), status );
  TRY( LALCreateNextDBEntry( status->statusPtr, output, "search_summary",
        "shared_object" ), status );
  dbase = *output;
  TRY( LALCreateDBEntryBLOB( status->statusPtr, dbase, blob, 1 ), status );
  TRY( LALDestroyBLOB( status->statusPtr, &blob ), status );

  TRY( LALCreateBLOB( status->statusPtr, &blob, lalWrapperCVSTag,
        strlen( lalWrapperCVSTag ) ), status );
  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "lalwrapper_cvs_tag" ), status );
  TRY( LALCreateDBEntryBLOB( status->statusPtr, dbase, blob, 1 ), status );
  TRY( LALDestroyBLOB( status->statusPtr, &blob ), status );

  TRY( LALCreateBLOB( status->statusPtr, &blob, lalCVSTag,
        strlen( lalCVSTag ) ), status );
  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "lal_cvs_tag" ), status );
  TRY( LALCreateDBEntryBLOB( status->statusPtr, dbase, blob, 1 ), status );
  TRY( LALDestroyBLOB( status->statusPtr, &blob ), status );

  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "in_start_time" ), status );
  TRY( LALCreateDBEntryData( status->statusPtr, dbase, int_4s, 1 ), status );
  dbase->rows.int4s[0] = info->inputStartTime.gpsSeconds;

  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "in_start_time_ns" ), status );
  TRY( LALCreateDBEntryData( status->statusPtr, dbase, int_4s, 1 ), status );
  dbase->rows.int4s[0] = info->inputStartTime.gpsNanoSeconds;

  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "in_end_time" ), status );
  TRY( LALCreateDBEntryData( status->statusPtr, dbase, int_4s, 1 ), status );
  dbase->rows.int4s[0] = info->inputEndTime.gpsSeconds;

  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "in_end_time_ns" ), status );
  TRY( LALCreateDBEntryData( status->statusPtr, dbase, int_4s, 1 ), status );
  dbase->rows.int4s[0] = info->inputEndTime.gpsNanoSeconds;

  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "out_start_time" ), status );
  TRY( LALCreateDBEntryData( status->statusPtr, dbase, int_4s, 1 ), status );
  dbase->rows.int4s[0] = info->outputStartTime.gpsSeconds;

  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "out_start_time_ns" ), status );
  TRY( LALCreateDBEntryData( status->statusPtr, dbase, int_4s, 1 ), status );
  dbase->rows.int4s[0] = info->outputStartTime.gpsNanoSeconds;

  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "out_end_time" ), status );
  TRY( LALCreateDBEntryData( status->statusPtr, dbase, int_4s, 1 ), status );
  dbase->rows.int4s[0] = info->outputEndTime.gpsSeconds;

  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "out_end_time_ns" ), status );
  TRY( LALCreateDBEntryData( status->statusPtr, dbase, int_4s, 1 ), status );
  dbase->rows.int4s[0] = info->outputEndTime.gpsNanoSeconds;

  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "nevents" ), status );
  TRY( LALCreateDBEntryData( status->statusPtr, dbase, int_4s, 1 ), status );
  dbase->rows.int4s[0] = info->numEvents;

  TRY( LALCreateNextDBEntry( status->statusPtr, &dbase, "search_summary",
        "nnodes" ), status );
  TRY( LALCreateDBEntryData( status->statusPtr, dbase, int_4s, 1 ), status );
  dbase->rows.int4s[0] = info->numNodes;

  DETATCHSTATUSPTR( status );
  RETURN( status );
}
